package org.w3.rdfvalidator

import org.scalatra._
import java.net.URL
import javax.servlet.http._

import dispatch._
import Http._
import scala.xml.{NodeSeq, Comment, Elem}

import org.w3.sw.rdf._
import org.w3.sw.rdfxml._

object RDFValidator {
  val rdfxmlParser = new org.w3.sw.rdfxml.RDFXML

  def toString(t:Triple):Elem = <li><span>{ toString(t.s) }</span> <span>{ toString(t.p) }</span> <span>{ toString(t.o) }</span></li>
  def toString(s:Subject):String = {
    val SubjectNode(n) = s
    toString(n)
  }
  def toString(p:Predicate):String = {
    val PredicateIRI(iri) = p
    toString(iri)
  }
  def toString(iri:IRI):String = {
    val IRI(i) = iri
    "<"+i+">"
  }
  def toString(bnode:BNode):String = {
    val BNode(label) = bnode
    "_:"+label
  }
  def toString(n:Node):String = {
    n match {
      case NodeIRI(iri)     => toString(iri)
      case NodeBNode(bnode) => toString(bnode)
    }
  }
  def toString(o:Object):String = o match {
    case ObjectNode(n) => toString(n)
    case ObjectLiteral(l) => toString(l)
  }
  def toString(l:Literal):String = l match {
    case PlainLiteral(lexicalForm, langtag) => "\"" + lexicalForm + "\"" + { if (langtag.isDefined) ("@"+langtag.get) else "" }
    case TypedLiteral(lexicalForm, datatype) => "\"" + lexicalForm + "\"^^" + toString(datatype)
  }

  def message(title:String, description:String):NodeSeq =
    <message type="info"> 
      <title>{ title }</title> 
      <description>{ description }</description> 
    </message> 

  def renderMessage(graph:Graph) =
    if (graph.nonEmpty)
      <message type="info"> 
        <title>Parsed triples</title> 
        <description>{ <ol>{ graph map { toString(_) } }</ol> }</description> 
      </message>
    else
      Comment(" no parsed triple ")

  def renderMessage(e:ParseError) = {
    val title = e.errorType match {
      case RDFParseError => "RDF issue"
      case XMLParseError => "XML issue"
    }
    val errorType = e.level match {
      case Warning            => "warning"
      case FatalError | Error => "error"
    }
    <message type={ errorType }> 
      <title>{ title }</title> 
      <description>{ e.message }</description> 
    </message> 
  }

}



class RDFValidator extends ScalatraServlet {

  import RDFValidator._

  get("/*", request.getParameter("uri") != null) {
    contentType = "text/xml"
    val uri = request.getParameter("uri")
    val http = new Http
    try {
      http(uri >> { in => {
        val (graph, parseErrors) = rdfxmlParser.toGraph(in)
        val statusValue =
          if (parseErrors exists { e => e.level == FatalError || e.level == Error })
            "failed"
          else
            "passed"
        <observationresponse xmlns="http://www.w3.org/2009/10/unicorn/observationresponse" xml:lang="en" ref={ uri }>
          <status value={ statusValue }/>
          { renderMessage(graph) }
          { parseErrors map { renderMessage(_) } }
        </observationresponse>
      }})
    } catch {
      case StatusCode(code, contents) => {
        <observationresponse xmlns="http://www.w3.org/2009/10/unicorn/observationresponse" xml:lang="en" ref={ uri }>
          <status value="failed"/>
          { message("HTTP "+code, contents) }
        </observationresponse>
      }
      case e:Exception => {
        e.printStackTrace
        <observationresponse xmlns="http://www.w3.org/2009/10/unicorn/observationresponse" xml:lang="en" ref={ uri }>
          <status value="failed"/>
          <message type="info"> 
            <title>Something wrong happened</title> 
            <description>{ e.getMessage }</description> 
          </message> 
        </observationresponse>
      }
    }
  }

  notFound {
    "notFound"
  }

}
